local _, c = UnitClass("player")
if c ~= "ROGUE" then return end

PoisonSwapper = LibStub("AceAddon-3.0"):NewAddon("PoisonSwapper", "AceEvent-3.0", "AceBucket-3.0")
local mod = PoisonSwapper
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")

-- With Rank
local deadlyName, _, icon = GetSpellInfo(57970)

-- Without rank
local instantName = GetSpellInfo(8680)
local woundName = GetSpellInfo(13219)

local tt = CreateFrame("GameTooltip", "PoisonSwapTooltip", UIParent, "GameTooltipTemplate")
tt:SetOwner(UIParent, "ANCHOR_NONE")

local ldbObj = LDB:NewDataObject("PoisonSwapper", {
	type = "data source",
	icon = icon,
	label = "PoisonSwapper",
	text  = "|cff00ff00Enabled|r",	
	OnClick = function(self, btn)
		mod:Toggle()
	end
})

local function scanFor(bag, slot, poison)
	tt:SetBagItem(bag, slot)
	for i = 1, tt:NumLines() do
		local l = _G["PoisonSwapTooltipTextLeft" .. i]
		if l then
			local text = l:GetText()
			if text then
				local poisonMatch = text:match(poison)
				if poisonMatch then
					return true
				end
			end
		end
	end
	return false
end

function mod:Equip(ofType)
	local location = self:FindWeapon(ofType)
	if location then
		EquipmentManager_EquipContainerItem(EquipmentManager_EquipItemByLocation(location, 17))
	end
end

function mod:HasImpPoisons()
	if self.hasImpPoisons ~= nil then return self.hasImpPoisons end
	local _, _, _, _, points = GetTalentInfo(1,11)
	self.hasImpPoisons = points >= 3
	return self.hasImpPoisons
end

local buffer = {}
function mod:CheckSwap(forceTo)
	local useDamage = false
	local changing, stacks, expires, name, unit = nil, 0, 0, nil, nil
	
	for i = 1, 40 do 
		name, _, _, stacks, _, _, expires, unit = UnitDebuff("target", i)
		if name == deadlyName and unit == "player" then 					
			break
		end
		if not name then break end
	end
	if expires then
		useDamage = expires - GetTime() > 5 and stacks == 5
	end
	
	if useDamage then
		if self:HasImpPoisons() then
			
			self:Equip(instantName)
		else
			self:Equip(woundName)
		end
	else
		self:Equip(deadlyName)
	end
end

function mod:FindWeapon(ofType)
	wipe(buffer)
	GetInventoryItemsForSlot(17, buffer)
	for location, itemID in pairs(buffer) do
		local player, bank, bags, slot, bag = EquipmentManager_UnpackLocation(location)
		if bags and bag then
			if scanFor(bag, slot, ofType) then
				return location
			end
		end
	end
	return nil
end

function mod:HandleEvent()
	if not PoisonSwapperEnabled then return end
	if not InCombatLockdown() or not UnitCanAttack("player", "target") then return end
	self:CheckSwap()
end

function mod:OnInitialize()
	if PoisonSwapperEnabled == nil then PoisonSwapperEnabled = true end
	self:SetEnabledState(PoisonSwapperEnabled)
	self:Toggle(PoisonSwapperEnabled)
	SLASH_POISONSWAPPER1 = "/ps"
	SlashCmdList["POISONSWAPPER"] = mod.Toggle
end

function mod:Toggle(state)
	PoisonSwapperEnabled = state == nil and (not PoisonSwapperEnabled) or state
	local color, state
	if PoisonSwapperEnabled then
		color, state = "ff00ff00", "Enabled"
	else
		color, state = "ffff0000", "Disabled"
	end
	ldbObj.text = ("|c%s%s|r"):format(color, state)
	print(("PoisonSwapper: |c%s%s|r"):format(color, state))
end

function mod:OnEnable()
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "HandleEvent")
	self:RegisterBucketEvent("UNIT_AURA", 0.1, "HandleEvent")
	self:RegisterEvent("CHARACTER_POINTS_CHANGED")
end

function mod:CHARACTER_POINTS_CHANGED()
	self.hasImpPoisons = nil
end